// 20.04.2024
// PowerMeter AD8310
// Ext ATT 50dB
//-------------
// UNO
// A4 ----> LCD SDA
// A5 ----> LCD SCL
// A0 ----> AD8310
// D2 ----> Key freeze LCD
// D3 ----> Key Band select

#include <Wire.h>
#include <LiquidCrystal_I2C.h>


int value = 0;           // Used to store the raw reading from the ADC
int buttonStateF = 0;
int buttonStateB = 0;
float vout = 0.000;      // Holds the true value of the ADC output voltage
float powerdB = 0.00;    // Calculated power in dBm
float pWatts = 0.00;     // Calculated power in watts
float slope = 40.8;      // Slope 40.8 - 145MHz; 43.8 - 435MHz
float intercept = 95;    // 0V intercept point
float refVolts = 5.02;   // Reference voltage
float attenuation = 50;  // Attenuator 50 dB
long lastTime = 0;
const int buttonF = 2;  // Button freeze LCD
const int buttonB = 3;  // Button band


LiquidCrystal_I2C lcd(0x27, 8, 2);

void setup() {
  pinMode(buttonF, INPUT);
  pinMode(buttonB, INPUT);
  digitalWrite(buttonF, HIGH);
  digitalWrite(buttonB, HIGH);
  lcd.init();
  lcd.backlight();
  lcd.setCursor(0, 0);
  lcd.print("RF Power");
  lcd.setCursor(2, 1);
  lcd.print("Meter ");
  delay(2000);
}

void loop() {

  if (millis() - lastTime > 250) {
    lastTime = millis();

    buttonStateF = digitalRead(buttonF);
    buttonStateB = digitalRead(buttonB);
    if (buttonStateB == HIGH) {
      slope = 40.8;
    } else {
      slope = 43.8;
    }

    if (buttonStateF == HIGH) {
      value = analogRead(0);  // Read the ADC and store the result in value
    } else {
      value;
    }

    vout = (value * refVolts) / 1023;                      // Convert the ADC result to volts in vout
    powerdB = (slope * vout) - (intercept - attenuation);  // convert the voltage to dBm in 50 ohms
    pWatts = pow(10.0, (powerdB - 30) / 10.0);             // convert dBm to watts
    lcd.setCursor(0, 0);
    lcd.print(pWatts, 1);
    lcd.print(" W");
    lcd.print("    ");
    lcd.setCursor(0, 1);
    lcd.print(powerdB, 1);
    lcd.print("dBm");
    lcd.print("   ");
  }
}